<?php
use Dedimania\Dedimania;
use FML\Script\Builder;
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\CallbackManager;
use ManiaControl\Formatter;
use ManiaControl\ManiaControl;
use ManiaControl\Maps\MapManager;
use ManiaControl\Players\Player;
use ManiaControl\Players\PlayerManager;
use ManiaControl\Plugins\Plugin;
use ManiaControl\Players;

class PersonalMessages implements CallbackListener, Plugin, \ManiaControl\Commands\CommandListener, \ManiaControl\Callbacks\TimerListener {
	/**
	 * Private properties
	 */
	/** @var maniaControl $maniaControl */
	private $maniaControl = null;

	private $replyTarget = array();

	private $animationManialink = null;
	private $pmManialink = null;
	private $pmHideManialink = null;

	/**
	 * Prepares the Plugin
	 *
	 * @param ManiaControl $maniaControl
	 * @return mixed
	 */
	public static function prepare(ManiaControl $maniaControl) {
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::load()
	 */
	public function load(ManiaControl $maniaControl) {
		$this->maniaControl = $maniaControl;

		// Register callbacks
		$this->maniaControl->callbackManager->registerCallbackListener(CallbackManager::CB_MP_PLAYERMANIALINKPAGEANSWER, $this, 'onManialinkPageAnswer');
		$this->maniaControl->callbackManager->registerCallbackListener(PlayerManager::CB_PLAYERDISCONNECT, $this, 'onPlayerDisconnect');

		// Register commands
		$this->maniaControl->commandManager->registerCommandListener('pm', $this, 'onCommandPm');
		$this->maniaControl->commandManager->registerCommandListener('pmr', $this, 'onCommandPmr');

		// Load manialink files
		$this->animationManialink = file_get_contents(__DIR__ . DIRECTORY_SEPARATOR . 'Animation.Script.txt', FILE_USE_INCLUDE_PATH);
		$this->pmManialink = file_get_contents(__DIR__ . DIRECTORY_SEPARATOR . 'PmManialink.xml', FILE_USE_INCLUDE_PATH);
		$this->pmHideManialink = '<manialink id="pm.widget"></manialink>';

		return true;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::unload()
	 */
	public function unload() {
		$this->maniaControl->callbackManager->unregisterCallbackListener($this);
		$this->maniaControl->commandManager->unregisterCommandListener($this);
		$this->maniaControl->timerManager->unregisterTimerListenings($this);
		unset($this->maniaControl);
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getId()
	 */
	public static function getId() {
		return 34;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getName()
	 */
	public static function getName() {
		return 'Personal Messages Plugin';
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getVersion()
	 */
	public static function getVersion() {
		return 1;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getAuthor()
	 */
	public static function getAuthor() {
		return 'TGYoshi';
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getDescription()
	 */
	public static function getDescription() {
		return 'Adds two commands to communicate through personal messages.';
	}

	/**
	 * Handles the player disconnection.
	 *
	 * @param Player $player
	 */
	public function onPlayerDisconnect(Player $player) {
		unset($this->replyTarget[$player->login]);
	}

	/**
	 * Handles the manialink page answer to send/hide the manialink after ui interaction.
	 *
	 * @param array $callback
	 */
	public function onManialinkPageAnswer(array $callback) {
		$call = $callback[1][2]; // Action
		$params = explode('|', $call, 3); // Split by |, always 3 entries; like:  PM|targetLogin|message with | and spaces

		$source = $this->maniaControl->playerManager->getPlayer($callback[1][1], true); // Get caller
		if (count($params) != 3 || $params[0] != 'PM' || $source === null) // Assertions
			return;

		// If we actually send a pm
		if ($params[1] != '!hide') { // If the target login is !hide, it isn't a real message
			$target = $this->maniaControl->playerManager->getPlayer($params[1], true);
			if ($target !== null) {
				// Send pm
				$this->sendPm($source, $target, $params[2]);
			}
		}

		// Hide the page after a short while (permit animation to finish)
		$self = $this;
		$this->maniaControl->timerManager->registerOneTimeListening($this, function($time) use (&$self, &$source){
			$self->hidePmManialink($source);
		}, 250);
	}

	/**
	 * Sends a personal message from $source to $target.
	 *
	 * @param Player $source
	 * @param Player $target
	 * @param        $message
	 */
	private function sendPm(Player $source, Player $target, $message) {
		$this->maniaControl->client->chatSendServerMessage('$08f[To $<' . $target->nickname . '$>] $ff0' . $message, $source->login);
		$this->maniaControl->client->chatSendServerMessage('$08f[From $<' . $source->nickname . '$>] $ff0' . $message, $target->login);
		// Store a reply reference to this player for quick access through /pmr
		$this->replyTarget[$target->login] = $source->login;
	}

	/**
	 * Handles the command /pm (send a pm to a specific user)
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function onCommandPm(array $chatCallback, Player $player) {
		$params = explode(' ', $chatCallback[1][2], 3); // Command: /pm <target> <message>
		if (count($params) >= 2) {
			if (trim($params[1]) == '') {
				// No login given
				$this->sendPmManialink($player, '');
			} else {
				// Login given, check if it exists
				$target = null;
				try {
					$target = $this->maniaControl->playerManager->getPlayer(trim($params[1]), true);
				} catch (Exception $e) {}

				if ($target === null)
					$this->maniaControl->chat->sendError("No connected player found with that login (use /pm without parameters to get a list of players).", $player->login);
				else if (count($params) === 2 || trim($params[2]) == '') // No message was passed; but player exists, ask for the message only
					$this->sendPmManialink($player, $target->login);
				else // Send instantly
					$this->sendPm($player, $target, $params[2]);
			}
		} else { // No login given
			$this->sendPmManialink($player, '');
		}
	}

	/**
	 * Handles the command /pmr (pm reply).
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function onCommandPmr(array $chatCallback, Player $player) {
		if (!isset($this->replyTarget[$player->login])) {
			$this->maniaControl->chat->sendError("Nobody to reply to (use /pm to send a new message).", $player->login);
			return;
		}
		$target = null;
		try {
			$target = $this->maniaControl->playerManager->getPlayer($this->replyTarget[$player->login], true);
		} catch (Exception $e) {}

		if ($target === null) {
			$this->maniaControl->chat->sendError("Reply target is no longer connected.", $player->login);
			return;
		}

		$params = explode(' ', $chatCallback[1][2], 2); // Command: /pmr <message>
		if (count($params) === 2 && trim($params[1]) != '') { // Message was given
			$this->sendPm($player, $target, $params[1]);
		} else { // No message given yet
			$this->sendPmManialink($player, $target->login);
		}
	}

	/**
	 * Opens the pm dialog manialink for a player.
	 * Passing an empty string to $targetlogin will ask the player to who, if it's a valid login it'll only ask for a message.
	 *
	 * @param Player $player
	 * @param string $targetLogin
	 */
	public function openPmDialog(Player $player, $targetLogin = '') {
		$this->sendPmManialink($player, $targetLogin);
	}

	/**
	 * Hides the pm dialog for a specific player.
	 *
	 * @param Player $target
	 */
	private function hidePmManialink(Player $target) {
		$this->maniaControl->client->sendDisplayManialinkPage($target->login, $this->pmHideManialink, 0, false);
	}

	/**
	 * Sends the pm manialink to a player.
	 * Passing an empty string to $targetlogin will ask the player to who, if it's a valid login it'll only ask for a message.
	 *
	 * @param Player $target
	 * @param        $targetLogin
	 */
	private function sendPmManialink(Player $target, $targetLogin) {

		$msPlayersArray = ""; // Maniascript players array ([login, nickname, other search variants...])
		$msPlayerFrames = ""; // Player frames xml

		foreach ($this->maniaControl->playerManager->getPlayers() as $player) {
			/** @var  Player $player */
			$plainNick = Builder::escapeText(preg_replace('/[^\x20-\x7E]/','', Formatter::StripCodes($player->nickname)));
			$msPlayersArray .= 'players.add(["' . Builder::escapeText($player->login) . '", "' . Builder::escapeText($player->nickname) . '", "' . $plainNick . '", "' . strtolower($plainNick) . '"]);';
			$msPlayerFrames .= '
				<frame posn="0 6" id="_player_' . Builder::escapeText($player->login) . '" valign="center">
				  <quad style="Bgs1" substyle="BgColorContour" sizen="30 2.6" id="_quad" opacity="0" posn="0 0 1" valign="center"/>
				  <quad sizen="30 2.6" bgcolor="345" opacity="0" class="hlop" valign="center"/>
				  <label posn="0.5 0.3 3" text="' . Builder::escapeText($player->nickname) . '" scale="0.8" textemboss="1" opacity="0" valign="center"/>
				  <label posn="29.5 0.3" text="$aaa$i' . Builder::escapeText($player->login) . '" scale="0.65" textemboss="1" halign="right" opacity="0" valign="center"/>
				</frame>';

		}

		$manialink = str_replace(array('{{animationsManialink}}', '{{msPlayerArray}}', '{{targetLogin}}', '{{msPlayerFrames}}'),
		                         array($this->animationManialink, $msPlayersArray, Builder::escapeText($targetLogin), $msPlayerFrames),
			                     $this->pmManialink);
		$this->maniaControl->client->sendDisplayManialinkPage($target->login, $manialink, 0, false);
	}
}
